﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=1		// Use modern global access method and strict wave access.


//CALCULATE THE KERNAL
function KernalV1(N_SP2,m_star,max_charge,mass_mob_pref,mass_mob_exp,omega,electric_charge_constant,V,flowrate_CPMA,Temperature,Pressure, T_neut,omega_hat)
	variable  max_charge,mass_mob_pref,mass_mob_exp,electric_charge_constant,flowrate_CPMA,Temperature,Pressure,T_neut,omega_hat
	wave N_SP2, m_star, omega, V
	//define constants
	variable tolerance = 0.0025
	variable step_size = 1.0025
	
	//define the CPMA dimensions - I need to make this a function at some point
	variable CPMA_gap, CPMA_r1, CPMA_r2, CPMA_rhat, CPMA_omega_hat, CPMA_rc, CPMA_L, CPMA_k
	//define the CPMA constants
	CPMA_gap   =   .001;             //  gap - distance between cylinders (m)
	CPMA_r1   =   .060;              // r1 - inner radius of APM (m)
	CPMA_r2   =   CPMA_r1+CPMA_gap;             // r2 - outer radius of APM (m)
	CPMA_rhat   =   CPMA_r1/CPMA_r2;   //r_hat
	CPMA_omega_hat   =   omega_hat;	//omega_hat
	CPMA_rc   =   (CPMA_r2+CPMA_r1)/2;       //   rc - the center radius
	CPMA_L   =   .200;              // L - length of electrode (cylinder) (m)
	CPMA_k   =   1.3806503*10^-23;  // k - boltzmann's constant
	
	variable i = 0 // counting variable
	make/D/O/N=(numpnts(m_star)+1) bin_limits=0
		//calculate the bins which correspond to the centres of the measured bins - i.e. the points in between m_stars
		for (i=0;i<numpnts(m_star)-1;i+=1)
			bin_limits[i+1] = sqrt(m_star[i]*m_star[i+1]) // Geometric midpoint = sqrt(upper*lower)
		endfor
	bin_limits[0] = m_star[0]^2/bin_limits[1];  //first bin
	bin_limits[numpnts(bin_limits)] = m_star[numpnts(m_star)]^2/bin_limits[numpnts(bin_limits)-2] //last bin
	//print bin_limits
	//determine if we need more bins above or below these limits because of multiply charged particles (above) or diffusion (below)
		//Add bins up top since no diffusion should occur (big particles)
	variable max_m_star =  calc_m_star(max_charge*electric_charge_constant,V[numpnts(V)],omega[numpnts(omega)]) //the highest mass occurs at charge state =max_charge
	variable m_max = calc_m_max(max_m_star,max_charge*electric_charge_constant,omega[numpnts(omega)],V[numpnts(V)],mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat)
	m_max*=1.1 //make it a bit higher because diffusion model is not perfect
	i = numpnts(bin_limits)
	variable original_bin_limits = i
	variable/G bins_added_on_right =0
	
	do 
		redimension/N=(numpnts(bin_limits)+1) bin_limits
		bin_limits[i]=bin_limits[numpnts(bin_limits)-2]^2/bin_limits[numpnts(bin_limits)-3]
		bins_added_on_right= i-original_bin_limits+1
		i+=1
	while (m_max>bin_limits[numpnts(bin_limits)])
	
	//need to make sure this works when particle diffusion is large and spills into the next bin
		//this adds bin to the lower end if diffusion spills into the next bin
		//first calcualte the difference between bin limits (logrithmic constant)
	make/O/N=(numpnts(bin_limits)-1) log_constant
	for (i=0;i<numpnts(bin_limits)-1;i+=1)
		log_constant[i] = ln(bin_limits[i+1]/bin_limits[i])
	endfor
	variable bin_constant  = median2(log_constant)
	
	//This computes the transfer function, omega, at the lowest bin_limtits, and checks if the transfer function is below a certain tolerance.
	//If the transfer function is not below the tolerance, then a new bin is added because diffusion caused the transfer function to broaden
	variable test =1
	make/D/O/N=1 B_temp
	make/D/O/N=1 trans
	//variable B_temp
	//variable trans
	variable count =1
	variable/G bins_added_on_left
	
	do 
		B_temp= mass2mob(bin_limits[0],mass_mob_pref,mass_mob_exp,Pressure,Temperature)
		trans = CPMA_diff_model_fixedgrid(bin_limits[0],B_temp ,electric_charge_constant,flowrate_CPMA,omega[0],V[0],Temperature,omega_hat)
		if (trans[0]<tolerance)
			test = 0
		else
			insertpoints 0,1,bin_limits
			bin_limits[0] = bin_limits[1]/exp(bin_constant)
		endif
		bins_added_on_left = count-1
		count +=1
	while  (test == 1)
	
	//need to make sure m_bin matches m_star, and we added bins so make new variable m_bin. m_star = m_bin
	make/D/O/N=(numpnts(bin_limits)-1) m_lower
	make/D/O/N=(numpnts(bin_limits)-1) m_upper
	make/D/O/N=(numpnts(bin_limits)-1) m_bin
	make/D/O/N=(numpnts(bin_limits)-1) d_m_bin
	
	for (i=0;i<numpnts(bin_limits)-1;i+=1)
		m_lower[i] = bin_limits[i]
		m_upper[i] = bin_limits[i+1]
		m_bin[i] = sqrt(bin_limits[i]*bin_limits[i+1])
		d_m_bin[i] = (m_bin[i]/mass_mob_pref)^(1/mass_mob_exp)
	endfor
	
	
	
								//////build the KERNAL/////
	make/O/N=(numpnts(N_SP2),numpnts(m_bin)) K=0
	variable size_transu =0
	variable Rm
	make/D/O/N=(numpnts(N_SP2),max_charge) m_star_mult_charge
	//make/D/O/N=(numpnts(N_SP2),max_charge) m_bin
	//make/D/O/N=(numpnts(N_SP2),max_charge) m_bin
	variable phi_count = 0
	variable phi = 1
	variable u=1 
	variable right_condition = 1
	variable left_condition = 1
	variable need_more_m_temps_on_right = 1
	variable need_more_m_temps_on_left = 1
	variable find_ind1
	variable bin_ind1
	variable find_ind2
	variable bin_ind2
	variable j=0
	variable case_test
	variable kk=0
	variable trans_int_lower
	variable trans_int_upper
	variable yy=0
	 i = 0
	 
	// phi_count=1 //For testing
	// i=5
	
	for (i=0;i<numpnts(N_SP2);i+=1)
		for (phi_count=0;phi_count<(max_charge);phi_count+=1)
		
		phi = phi_count+1
		
		
		//calculate the m_star consdidering the charge state (phi=1 to max_charge, where max_charge usually equals 3)
		 
		m_star_mult_charge[i][phi_count] =  calc_m_star(phi*electric_charge_constant,V[i],omega[i]) //the highest mass occurs at charge state =max_charge 
		//must reset these variables 
		u=0
		right_condition=1
		left_condition=1
		need_more_m_temps_on_right = 1 
		need_more_m_temps_on_left = 1
		
		//we will need to integrate the transfer function of each m_star so need to re-create the transfer function to integrate. We will make m_temps by considering a minimum vlaue of the transfer fucntion
		//if trans(u) < tolerance than stop making m_temps. Do this for both sides of m_Star. There is probably a more efficiency way to do this
		make/D/O/N=1 m_temp =m_star_mult_charge[i][phi_count] //the first m_temp is at the middle of the CPMA transfer function = m_star(i.phi)
		Redimension/N=1 B_temp //need to make sure B_temp starts with one value 
		B_temp = mass2mob(m_temp[0],mass_mob_pref,mass_mob_exp,Pressure,Temperature)  //for the first m_temp
		Redimension/N=1 trans
		trans = CPMA_diff_model_fixedgrid(m_temp[0],B_temp[0] ,phi*electric_charge_constant,flowrate_CPMA,omega[i],V[i],Temperature,omega_hat) //for the first m_temp, not necessarily 1
		
		//first deal with the right hand side. Trans is strictly decreasing as it gets farther away from m_star in both directions
		u=1
		variable placeholder =1
		do	
			redimension/N=(numpnts(m_temp)+1) m_temp
			redimension/N=(numpnts(B_temp)+1) B_temp
			redimension/N=(numpnts(trans)+1) trans
			 m_temp[u] = m_temp[u-1]*step_size
			 B_temp[u] = mass2mob(m_temp[u],mass_mob_pref,mass_mob_exp,Pressure,Temperature)
			 trans[u] = CPMA_diff_model_fixedgrid(m_temp[u],B_temp[u] ,phi*electric_charge_constant,flowrate_CPMA,omega[i],V[i],Temperature,omega_hat)
			 
			if (trans[u]<tolerance)
				//need to delete the last point calculated if trans<tolerance
				deletepoints u,1,m_temp
				deletepoints u,1,B_temp
				deletepoints u,1,trans
				need_more_m_temps_on_right=0
			endif
			u+=1
		while (need_more_m_temps_on_right==1)
		
		u=0 //reset the counter
		//now we are adding trans values to the left
		
		do
			insertpoints 0,1,m_temp
			insertpoints 0,1,B_temp
			insertpoints 0,1,trans
			m_temp[0] = m_temp[1]/step_size
			B_temp[0] = mass2mob(m_temp[0],mass_mob_pref,mass_mob_exp,Pressure,Temperature)
			trans[0] = CPMA_diff_model_fixedgrid(m_temp[0],B_temp[0] ,phi*electric_charge_constant,flowrate_CPMA,omega[i],V[i],Temperature,omega_hat)
			if (trans[0]<tolerance)
				//need to delete the last point calculated if trans<tolerance
				deletepoints 0,1,m_temp
				deletepoints 0,1,B_temp
				deletepoints 0,1,trans
				need_more_m_temps_on_left=0
			endif
			u+=1
		while (need_more_m_temps_on_left==1)
		//edit m_temp
		//find the first bin the transfer function will go into
			//can use the following functions to find things in IGOR
			//findvalue
			//findlevels
			//extract/INDX
		
		extract/O/INDX  m_lower,find_ind1_temp, m_lower>=m_temp[0]
		find_ind1 = find_ind1_temp[0]
		//print find_ind1
		
		if (find_ind1==0)
			bin_ind1=0
		else
			bin_ind1 = find_ind1-1
		endif
		//find the last bin the transfer function will go into
		extract/O/INDX m_upper,find_ind2_temp,m_upper>=m_temp[numpnts(m_temp)]
		//print find_ind2_temp
		bin_ind2 = find_ind2_temp[0]
		//print bin_ind1
		//print bin_ind2
		//print phi
		if (bin_ind1==bin_ind2)
			make/D/O/N=(numpnts(trans)) trans_norm=trans/m_temp
			K[i][bin_ind1]+=eq_charge_frac(d_m_bin[bin_ind1],phi,T_neut)*areaxy(m_temp,trans_norm)
			//print 10
		else //if the transfer function is spread across multiple bins then integrate each bin
			
//			make/O/N=5 testx = {0.1e-18,.101e-18,5e-18,6e-18,10e-18}
//			make/O/N=5 testy = {.1,.2,.3,.2,0.7}
//			make/O/N=1 testx1 = {0.102e-18}
//			make/O/N=1 test_interp
//			test_interp = interp(testx1,testx,testy)
//			print test_interp
			
			for (j=bin_ind1;j<=bin_ind2;j+=1)
			make/D/O/N=0 new_m_temp
			make/D/O/N=0 new_trans
			make/D/O/N=1 trans_int
				if (m_temp[0]>=m_lower[j]) //case when the lower part of the transfer function is partially in a bin
					
					trans_int[0] =interp(m_upper[j],m_temp,trans)

					extract/O/INDX m_temp,ind,m_temp>m_upper[j]
					
					redimension/N=(ind[0]+1) new_m_temp
					redimension/N=(ind[0]+1) new_trans
					for (kk=0;kk<=(ind[0]-1);kk+=1)
						new_m_temp[kk]= m_temp[kk]
						new_trans[kk] = trans[kk]
					endfor
					new_m_temp[numpnts(new_m_temp)]= m_upper[j]
					new_trans[numpnts(new_trans)] = trans_int
					case_test=1
					//print case_test
					//print new_m_temp
					//print new_trans
					//print trans
					
					
				elseif (m_temp[numpnts(m_temp)]<m_upper[j]) //case when the upper part of the transfer function is partially in a bin
					
					trans_int =interp(m_lower[j],m_temp,trans)
					extract/O/INDX m_temp,ind,m_temp>m_lower[j]
					
					redimension/N=(ind[0]) new_m_temp
					redimension/N=(ind[0]) new_trans
					new_m_temp[0] = m_lower[j]
					new_trans[0] = trans_int[0]
					yy = ind[0]-1
					for (kk=1;kk<(numpnts(m_temp));kk+=1)
						new_m_temp[kk]= m_temp[yy]
						new_trans[kk]= trans[yy]
						yy+=1
					endfor
					case_test=2
					//print case_test
					
				else //case when a middle part of the transfer function is in a bin
					trans_int_lower =interp(m_lower[j],m_temp,trans)
					trans_int_upper =interp(m_upper[j],m_temp,trans)
					
					extract/O/INDX m_temp,ind1,m_temp>m_lower[j]
					extract/O/INDX m_temp,ind2,m_temp>m_upper[j]
					
					redimension/N=(ind1[0]+ind2[0]-2) new_m_temp
					redimension/N=(ind1[0]+ind2[0]-2) new_trans
	
					yy = ind1[0]-1
					for (kk=1;kk<=(numpnts(new_m_temp)-2);kk+=1)
						new_m_temp[kk]= m_temp[yy]
						new_trans[kk]= trans[yy]
						yy+=1
					endfor
					new_m_temp[0] = m_lower[j]
					new_m_temp[numpnts(new_m_temp)] = m_upper[j]
					new_trans[0]= trans_int_lower
					new_trans[numpnts(new_trans)]= trans_int_upper
					case_test=3
					//print case_test
					
				endif
			make/O/N=(numpnts(new_trans)) temp_ratio_new_trans_temp = new_trans/new_m_temp	
			K[i][j] = K[i][j]+eq_charge_frac(d_m_bin[j],phi,T_neut)*areaxy(new_m_temp,temp_ratio_new_trans_temp)
			//print K[i][j]
			//print eq_charge_frac(d_m_bin[j],phi,T_neut)
			//print areaxy(new_m_temp,temp_ratio_new_trans_temp)

			endfor
		
		//print phi
		endif
		
		endfor
		print i
		//print K
	endfor
	//display trans vs m_temp
	//setaxis left 0,1
	//Display K
	//wave KOut
	//KOut=K
	duplicate/O K KOut
//Edit K
//Edit KOut
	//return KOut

end