﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=1		// Use modern global access method and strict wave access.


// load in the data
function load_CPMA_data(headerInfo)
	STRUCT CPMAHeaderInfo& headerInfo
	// Load in the CPMA  data
	LoadWave/J/M/U={0,0,1,0}/D/E=1/K=0/L={7,8,0,0,0}"Matrix"
	NewPath/O cpath, S_path
	ReadCPMAHeader("cpath", S_fileName , headerInfo)
	Rename wave0,Matrix
	//we must sort the matrix so the setpoints are going in ascending order
	MDsort(Matrix,2)
end

// Find metadata inbetween scans and delete it
Function MetaFinder()
	wave Matrix
	variable i
	for(i=0;i<dimsize(Matrix,0);i+=1)
		if(Numtype(Matrix[i][0])==2)
		DeletePoints/M=0 i,1,matrix
		i-=1
		endif
	endfor
End

function reassign_Matrix(Matrix)
	wave Matrix
	variable num_rows  = dimsize(Matrix,0)
	make/O/N=(num_rows,1) ref_timing, m_star, N_CPMA, Rm_file,omega,V,Temp, N_SP2
	ref_timing = Matrix[p][1]
	m_star = Matrix[p][2]
	N_CPMA = Matrix[p][10]
	Rm_file= Matrix[p][8]
	omega= Matrix[p][13]
	V= Matrix[p][14]
end

//need to figure out how to load these
//flowrate = M[p][2]
//Temp = M[p][2]
//Pressure = M[p][2]
//omega_hat = M[p][2]

//Load the SP2 data
killwaves SP2_raw_data0

LoadWave/O/J/M/D/N=SP2_data_import/K=0/L={0,1,0,0,0} "Macintosh HD:Users:KurtisBroda:Google Drive:SP2 UAlberta experiment:Time_Mass_CSV:Filtered_20151208_41_65_Time_Mass.csv"
variable datew = date2secs(2015,12,08)

//import SP2 data as a matrix and run this function on it
function/WAVE load_SP2_data(SP2_data_import,datew)
	variable datew
	wave SP2_data_import
	
	
	
	duplicate/O/r=[][] SP2_data_import, SP2_raw_data
	
	MDsort(SP2_raw_data,1) // sort the data
	
	SP2_raw_data[][1] -=datew //remove date from SP2 data
	SP2_raw_data[][1] += 30 //time shift so they are equal
	
	return SP2_raw_data
end


////SP2_raw_data[][1] -= 15
//duplicate/O/r=[][1] SP2_raw_data SP2_time
//
//display SP2_raw_data[][0] vs SP2_time
//AppendToGraph m_star_step vs CPMA_scan_time
//ModifyGraph mode(SP2_raw_data)=2,rgb(m_star_step)=(4369,4369,4369)
//ModifyGraph tick=2,mirror=1
 
//insert scan number
Function ScanNoFinder()
	wave wave0
	make/o/n=(numpnts(wave0)) ScanNumber=1
	variable i,j=(numpnts(wave0))-1
	For(i=0;i<j;i+=1)
		If(wave0[i+1]<wave0[i])
	 	print i
		ScanNumber[i+1,j]+=1
		endif
	Endfor
End


Structure CPMAHeaderInfo
	Variable dt					// Date/time in Igor date/time format (seconds since 1904-01-01)
	Variable densityFactor, fractalIndex, sampleFlow
	Variable innerRatio, outerRatio, referenceTemperature, referencePressure
	Variable massClass, sizeClass, transitTime, extraDelayTime, averagingPeriod
	Variable startMass, endMass, startSize, endSize 
EndStructure


Function ReadCPMAHeader(pathName, fileName, headerInfo)
	String pathName		// Name of symbolic path or "" to display dialog.
	String fileName			// Name of file or "" to display dialog. Can also be full or partial path relative to symbolic path.
	STRUCT CPMAHeaderInfo& headerInfo	// Output
 
	Variable refNum
 
 	String fileFilters = "CPMA Files (*.cpma):.cpma;"	//display CPMA files only
 
	Open /F=fileFilters/P=$pathName /R refNum as fileName
	
	String formatStr
	String text

	FReadLine refNum, text
	string/g SerialInfo
	SerialInfo=text
	print SerialInfo
	
	FReadLine refNum, text
	formatStr = "Time:\t%d:%d:%d\tDate:\t%d-%d-%d"
	Variable hour, minute, second, year, month, day
	sscanf text, formatStr, hour, minute, second, year, month, day
	headerInfo.dt = Date2Secs(year,month,day) + 3600*hour + 60*minute + second
	
	FReadLine refNum, text
	formatStr = "Density factor:\t%g\tFractal index:\t%g\tSample flow (lpm):\t%g"
	Variable densityFactor, fractalIndex, sampleFlow
	sscanf text, formatStr, densityFactor, fractalIndex, sampleFlow
	headerInfo.dt = Date2Secs(year,month,day) + 3600*hour + 60*minute + second
	headerInfo.densityFactor = densityFactor
	headerInfo.fractalIndex = fractalIndex
	headerInfo.sampleFlow = sampleFlow

	FReadLine refNum, text
	formatStr = "Inner ratio:\t%g\tOuter ratio:\t%g\tReference Temperature (K):\t%g\tReference Pressure (Pa):\t%g"
	Variable innerRatio, outerRatio, referenceTemperature, referencePressure 
	sscanf text, formatStr, innerRatio, outerRatio, referenceTemperature, referencePressure
	headerInfo.innerRatio = innerRatio
	headerInfo.outerRatio = outerRatio
	headerInfo.referenceTemperature = referenceTemperature
	headerInfo.referencePressure = referencePressure 
	
	FReadLine refNum, text
	formatStr = "Mass class/dec:\t%g\tSize class/dec:\t%g\tTransit time (s):\t%g\tExtra delay time (s):\t%g\tAveraging period (s):\t%g"
	Variable massClass, sizeClass, transitTime, extraDelayTime, averagingPeriod 
	sscanf text, formatStr, massClass, sizeClass, transitTime, extraDelayTime, averagingPeriod
	headerInfo.massClass = massClass
	headerInfo.sizeClass = sizeClass
	headerInfo.transitTime = transitTime
	headerInfo.extraDelayTime = extraDelayTime
	headerInfo.averagingPeriod = averagingPeriod

	FReadLine refNum, text
	formatStr = "Start mass (fg):\t%g\tEnd mass< (fg):\t%g\tStart size (nm):\t%g\tEnd size< (nm):\t%g"
	Variable startMass, endMass, startSize, endSize 
	sscanf text, formatStr, startMass, endMass, startSize, endSize
	headerInfo.startMass = startMass
	headerInfo.endMass = endMass
	headerInfo.startSize = startSize
	headerInfo.endSize = endSize
	
	variable/g ScanTimeV
	ScanTimeV=headerInfo.dt
	String ScanTime=Secs2Time(ScanTimeV,3)
	Print "Start of scan:  "+ScanTime
	Close refNum
	return 0
End


Function CPMAHeaderLoader()
	String pathName = ""//C:Users:mirwin:Desktop:"
	String fileName = ""//20151208_CHAM1001.cmpa"
	
	Print "Today = ",date()," at ",time()
	
	variable/g densityFactor, fractalIndex
	variable/g sampleFlow,innerRatio, outerRatio, referenceTemperature, referencePressure
	variable/g massClass, sizeClass, transitTime, extraDelayTime, averagingPeriod,startMass
	variable/g endMass, startSize, endSize 
	STRUCT CPMAHeaderInfo headerInfo
	ReadCPMAHeader(pathName, fileName, headerInfo)
	densityFactor=headerInfo.densityFactor
 	fractalIndex=headerInfo.fractalIndex
	sampleFlow=headerInfo.sampleFlow
 	innerRatio=headerInfo.innerRatio
 	outerRatio=headerInfo.outerRatio
 	referenceTemperature=headerInfo.referenceTemperature
 	referencePressure=headerInfo.referencePressure
 	massClass=headerInfo.massClass
 	sizeClass=headerInfo.sizeClass
 	transitTime=headerInfo.transitTime
 	extraDelayTime=headerInfo.extraDelayTime
 	averagingPeriod=headerInfo.averagingPeriod
 	startMass=headerInfo.startMass
 	endMass=headerInfo.endMass
 	startSize=headerInfo.startSize
 	endSize=headerInfo.endSize
End


Window Gizmo0() : GizmoPlot
	PauseUpdate; Silent 1		// building window...
	// Building Gizmo 7 window...
	NewGizmo/W=(35,45,550,505)
	ModifyGizmo startRecMacro=700
	ModifyGizmo scalingOption=63
	AppendToGizmo Surface=root:dN2_dlog_mp_rBC,name=surface0
	ModifyGizmo ModifyObject=surface0,objectType=surface,property={ srcMode,0}
	ModifyGizmo ModifyObject=surface0,objectType=surface,property={ surfaceCTab,Rainbow}
	AppendToGizmo Axes=boxAxes,name=axes0
	ModifyGizmo ModifyObject=axes0,objectType=Axes,property={-1,axisScalingMode,1}
	ModifyGizmo ModifyObject=axes0,objectType=Axes,property={-1,axisColor,0,0,0,1}
	ModifyGizmo ModifyObject=axes0,objectType=Axes,property={0,ticks,3}
	ModifyGizmo ModifyObject=axes0,objectType=Axes,property={1,ticks,3}
	ModifyGizmo ModifyObject=axes0,objectType=Axes,property={2,ticks,3}
	ModifyGizmo modifyObject=axes0,objectType=Axes,property={-1,Clipped,0}
	ModifyGizmo setDisplayList=0, object=surface0
	ModifyGizmo setDisplayList=1, object=axes0
	ModifyGizmo autoscaling=1
	ModifyGizmo currentGroupObject=""
	ModifyGizmo showInfo
	ModifyGizmo infoWindow={462,23,1279,320}
	ModifyGizmo endRecMacro
	ModifyGizmo SETQUATERNION={0.527490,-0.119307,-0.177356,0.822233}
EndMacro
